SET NOCOUNT ON;
go
USE ImEx;
go
IF EXISTS
 (SELECT *
  FROM   sys.objects
  WHERE  object_id = OBJECT_ID(N'dbo.Baseline')
  AND    type IN (N'P', N'PC'))
BEGIN;
  DROP PROCEDURE dbo.Baseline;
END;
go
CREATE PROCEDURE dbo.Baseline
                 @MaxCandidatesPerSession smallint
AS
-- Declare variables to hold fields read from cursor
DECLARE @Year          smallint
       ,@Quarter       tinyint
       ,@SubjCode      char(3)
       ,@NumCandidates smallint
-- Declare variables to detect year/quarter change
       ,@CurYear       smallint
       ,@CurQuarter    tinyint
-- Declare variables for #sessions in a quarter and for session to add registration to
       ,@NumSessions   smallint
       ,@SessionNo     smallint;

-- Init year/quarter change variables
SET @CurYear     = 0;
SET @NumSessions = 0;

-- Declare and open cursor to read registrations
DECLARE RegistrationsCursor CURSOR LOCAL FORWARD_ONLY STATIC READ_ONLY
FOR SELECT   Year, Quarter, SubjCode, NumCandidates
    FROM     dbo.Registrations
    ORDER BY Year, Quarter;

OPEN RegistrationsCursor;

-- Fetch first row
FETCH NEXT
FROM  RegistrationsCursor
INTO  @Year, @Quarter, @SubjCode, @NumCandidates;

-- Process all rows
WHILE @@FETCH_STATUS = 0
BEGIN;

  -- If year or quarter changes, reset session number
  IF @Year    <> @CurYear
  OR @Quarter <> @CurQuarter
  BEGIN;
    SET @NumSessions = 0;
    SET @CurYear     = @Year;
    SET @CurQuarter  = @Quarter;
  END;

  -- Find a session with room for this registration
  SET @SessionNo = (SELECT TOP 1 SessionNo
                    FROM   dbo.Sessions
                    WHERE  Year       = @Year
                    AND    Quarter    = @Quarter
                    AND    SpaceLeft >= @NumCandidates);

  -- If no session found, create a new one
  IF @SessionNo IS NULL
  BEGIN;
    SET @NumSessions = @NumSessions + 1;
    SET @SessionNo   = @NumSessions;
    INSERT INTO dbo.Sessions (Year, Quarter, SessionNo, SpaceLeft)
    VALUES (@Year, @Quarter, @SessionNo, @MaxCandidatesPerSession);
  END;

  -- Now add current regsitration to selected session
  UPDATE dbo.Registrations
  SET    SessionNo = @SessionNo
  WHERE  Year      = @Year
  AND    Quarter   = @Quarter
  AND    SubjCode  = @SubjCode;

  UPDATE dbo.Sessions
  SET    SpaceLeft = SpaceLeft - @NumCandidates
  WHERE  Year      = @Year
  AND    Quarter   = @Quarter
  AND    SessionNo = @SessionNo;

  -- Fetch next row
  FETCH NEXT
  FROM  RegistrationsCursor
  INTO  @Year, @Quarter, @SubjCode, @NumCandidates;
END;

-- Clean up resources used by cursor
CLOSE      RegistrationsCursor;
DEALLOCATE RegistrationsCursor;
go
