SET NOCOUNT ON;
go
USE ImEx;
go

-- Fill Entries table with a large amount of random data.
-- Note: the creation of random entries is not optimized!!
--
-- The variables below govern the amount and distribution of data to be generated

TRUNCATE TABLE dbo.Registrations;

DECLARE @NumYears   tinyint  -- Number of years (times 4 quarters)
       ,@NumSubj    smallint -- Number of subjects (= rows) in each quarter - max 17500 (for more, change INSERT stmt below)
       ,@IncludeQ1  char(1)  -- Include data for quarter 1? (normal distribution: 100% 1-100, evenly distributed)
       ,@IncludeQ2  char(1)  -- Include data for quarter 2? (more small sessions:  70% 1-25, 30% 26-100)
       ,@IncludeQ3  char(1)  -- Include data for quarter 3? (more large sessions:  20% 1-40, 80% 41-100)
       ,@IncludeQ4  char(1); -- Include data for quarter 4? (bell curve distribution: 100% 4-80 with most around 42)
SELECT  @NumYears   = 10
       ,@NumSubj    = 1000
       ,@IncludeQ1  = 'Y'
       ,@IncludeQ2  = 'Y'
       ,@IncludeQ3  = 'Y'
       ,@IncludeQ4  = 'Y';
-- Loop counters
DECLARE @y tinyint, @q tinyint, @s smallint;
-- The line below ensures reproducible results - remove it to get truly random results
SET @y = RAND(123); -- Replace 123 with another seed value to get a different set of reproducible results

SET         @y = 1;
WHILE       @y <= @NumYears
BEGIN;
 SET        @q = 1;
 WHILE      @q <= 4
 BEGIN;
 -- Skip excluded quarters
  IF       (@q = 1 AND @IncludeQ1 = 'Y')
  OR       (@q = 2 AND @IncludeQ2 = 'Y')
  OR       (@q = 3 AND @IncludeQ3 = 'Y')
  OR       (@q = 4 AND @IncludeQ4 = 'Y')
  BEGIN;
   SET      @s = 1;
   WHILE    @s <= @NumSubj
   BEGIN;
    -- Random number generation - different expression for each quarter
    INSERT  dbo.Registrations
           (Year, Quarter,
            SubjCode,
            NumCandidates)
    VALUES (@y + 1994, @q,
            CHAR (@s / 676 + 65) + CHAR ((@s / 26) % 26 + 65) + CHAR (@s % 26 + 65),
            CASE @q
            WHEN 1 THEN CEILING(100 * RAND())
            WHEN 2 THEN CASE WHEN RAND() < 0.7 THEN CEILING(25 * RAND()) ELSE CEILING (75 * RAND()) + 25 END
            WHEN 3 THEN CASE WHEN RAND() < 0.2 THEN CEILING(40 * RAND()) ELSE CEILING (60 * RAND()) + 40 END
            WHEN 4 THEN CEILING(20 * RAND()) + CEILING(20 * RAND()) + CEILING(20 * RAND()) + CEILING(20 * RAND())
            END);
    SET     @s = @s + 1;
   END;
  END;
  SET       @q = @q + 1;
 END;
 SET        @y = @y + 1;
END;
go

-- Check distribution for each quarter
SELECT   Quarter, AVG(1.0 * NumCandidates) AS AvgNumCandidates
FROM     dbo.Registrations
GROUP BY Quarter
ORDER BY Quarter;
SELECT   NumCandidates, [1] AS Q1, [2] AS Q2, [3] AS Q3, [4] AS Q4
FROM    (SELECT NumCandidates, Quarter, SubjCode
         FROM   dbo.Registrations) p
  PIVOT (COUNT(SubjCode) FOR Quarter IN ([1],[2],[3],[4])) pvt
ORDER BY NumCandidates;
